package xyz.twovb.sgm.games.impl.capturethebrick;

/*
 * Created by 2vb - 3/7/2024
 */

import org.bukkit.Bukkit;
import org.bukkit.Location;
import org.bukkit.World;
import org.bukkit.command.CommandSender;
import org.bukkit.configuration.ConfigurationSection;
import org.bukkit.configuration.file.YamlConfiguration;
import org.bukkit.entity.Player;
import org.bukkit.event.EventHandler;
import org.bukkit.event.entity.PlayerDeathEvent;
import org.bukkit.event.player.PlayerQuitEvent;
import xyz.twovb.sgm.SGM;
import xyz.twovb.sgm.games.GameManager;
import xyz.twovb.sgm.games.Minigame;
import xyz.twovb.sgm.levels.LevelManager;
import xyz.twovb.toolbox.utils.ChatUtils;

import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.UUID;

public class CTB implements Minigame {
    final static String name = "CaptureTheBrick";
    private final UUID uuid;
    private final List<Player> players;
    private int bricks;
    private World gameWorld;
    private GameState state;

    public CTB() {
        this.uuid = UUID.randomUUID();
        this.state = GameState.PRESTART;
        this.players = new ArrayList<>();
    }

    public void init(CommandSender owner, String world) throws IOException {
        // Perform initialization logic here
        Bukkit.getPluginManager().registerEvents(this, SGM.getInstance());
        gameWorld = createGameWorld(world, uuid);
        gameWorld.setAutoSave(false);
        if (applyOptions()) {
            state = GameState.READY;
            owner.sendMessage("A new " + name + " instance with ID " + uuid + " has been created and is now ready!");
        } else {
            owner.sendMessage(ChatUtils.translate(SGM.getInstance().getMessages().getString("sgm.game.cant-start")));
        }
    }

    private boolean applyOptions() {
        File configFile = new File(LevelManager.gamePath + name.toLowerCase() + "/options.yml");
        YamlConfiguration config = YamlConfiguration.loadConfiguration(configFile);

        if (!configFile.exists()) {
            SGM.getInstance().getCLogger().log("options.yml does not exist at " + configFile.getAbsolutePath());
            return false;
        }

        // Process teams section
        List<String> teamsToProcess = Arrays.asList("red", "blue");

        for (String teamName : teamsToProcess) {
            ConfigurationSection team = config.getConfigurationSection("teams." + teamName);
            if (team != null) {
                List<String> brickSpawns = team.getStringList("brickSpawns");
                List<String> playerSpawnArea = team.getStringList("playerSpawnArea");

                // Log or process team-specific values
                SGM.getInstance().getCLogger().log(teamName + " Player Spawn Area: " + playerSpawnArea);

                // Log each team's brick spawns
                for (int i = 0; i < brickSpawns.size(); i++) {
                    SGM.getInstance().getCLogger().log(teamName + " Brick Spawn " + (i + 1) + ": " + brickSpawns.get(i));
                    // Process each brick spawn here if needed
                }
            }
        }

        // Process terrainInfo section
        ConfigurationSection terrainInfoSection = config.getConfigurationSection("terrainInfo");
        if (terrainInfoSection != null) {
            int territory = terrainInfoSection.getInt("territory");
            int boundary = terrainInfoSection.getInt("boundary");

            // Log or process terrainInfo values
            SGM.getInstance().getCLogger().log("Territory: " + territory);
            SGM.getInstance().getCLogger().log("Boundary: " + boundary);
        }

        // Additional configuration values outside of teams and terrainInfo
        String game = config.getString("game");
        int brickCount = config.getInt("brickCount");

        // Log or process additional configuration values
        SGM.getInstance().getCLogger().log("Game: " + game);
        SGM.getInstance().getCLogger().log("Brick Count: " + brickCount);

        return true;
    }

//    private void placeBrick(Integer integer) {
//
//    }

    @Override
    public void start() {
        if (state == GameState.READY) {
            state = GameState.STARTED;
            sendMessageToAllPlayers("The game has started!");
            for (Player player : players) {
                player.teleport(player.getWorld().getSpawnLocation());
            }
            Bukkit.getScheduler().runTaskTimer(SGM.getInstance(), this::onTick, 0L, 1L);
        }
    }

    @Override
    public void stop() {
        // Stop game logic
        state = GameState.ENDING;
        Player winner;
        if (players.size() <= 1) {
            winner = players.get(0);
            winner.sendMessage("Congratulations on winning the game!");
        }
        players.clear(); // Clear player list
        GameManager.getActiveGames().remove(uuid);
    }

    @Override
    public void addPlayer(Player player) {
        players.add(player);
        player.teleport(gameWorld.getSpawnLocation());
        player.teleport(new Location(gameWorld, 0, 0, 0));
        sendMessageToAllPlayers(player.getName() + " has joined the game!");
    }

    @Override
    public void removePlayer(Player player) {
        players.remove(player);
    }

    @EventHandler
    void onPlayerQuit(PlayerQuitEvent event) {
        Player player = event.getPlayer();
        if (players.contains(player)) {
            removePlayer(event.getPlayer());
        }
    }

    @EventHandler
    void onPlayerDeath(PlayerDeathEvent event) {
        Player player = event.getPlayer();
        if (players.contains(player) && state == GameState.STARTED) {
            sendMessageToAllPlayers(player.getName() + " was eliminated!");
            event.setDeathMessage("");
            removePlayer(player);
        }
    }

    @Override
    public List<Player> getPlayers() {
        return players;
    }

    @Override
    public String getName() {
        return name;
    }

    @Override
    public UUID getGameId() {
        return uuid;
    }

    @Override
    public GameState getState() {
        return state;
    }

    @Override
    public void onTick() {
        if (state == GameState.STARTED && players.size() <= 1) {
            this.stop();
        }
    }

}